﻿using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PrecastConcretePlantDatabaseImplement.Implements
{
    public class ClientStorage : IClientStorage
    {

        public ClientViewModel? Delete(ClientBindingModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            var res = context.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (res != null)
            {
                context.Clients.Remove(res);
                context.SaveChanges();
            }
            return res?.GetViewModel;
        }

        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            if (model.Id.HasValue)
                return context.Clients.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
            if (model.Email != null && model.Password != null)
                return context.Clients
                    .FirstOrDefault(x => x.Email.Equals(model.Email)
                                      && x.Password.Equals(model.Password))
                    ?.GetViewModel;
            if (model.Email != null)
                return context.Clients.FirstOrDefault(x => x.Email.Equals(model.Email))?.GetViewModel;
            return null;
        }

        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (model == null)
            {
                return new();
            }
            if (model.Id.HasValue)
            {
                var res = GetElement(model);
                return res != null ? new() { res } : new();
            }
            if (model.Email != null)
            {
                using var context = new PrecastConcretePlantDataBase();
                return context.Clients
                    .Where(x => x.Email.Contains(model.Email))
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }

        public List<ClientViewModel> GetFullList()
        {
            using var context = new PrecastConcretePlantDataBase();
            return context.Clients.Select(x => x.GetViewModel).ToList();
        }

        public ClientViewModel? Insert(ClientBindingModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            var res = Client.Create(model);
            if (res != null)
            {
                context.Clients.Add(res);
                context.SaveChanges();
            }
            return res?.GetViewModel;
        }

        public ClientViewModel? Update(ClientBindingModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            var res = context.Clients.FirstOrDefault(x => x.Id == model.Id);
            res?.Update(model);
            context.SaveChanges();
            return res?.GetViewModel;
        }
    }
}
